##
# This program is a wrapper class, launching interactive text-based UI to map
# and analyze small RNAs to the C. elegans genome using a Galaxy bioinformatics
# cluster. Galaxy workflows were created by Andrew Blumenfeld and Antony Jose
# in http://www.clfs.umd.edu/cbmg/joselab/myPdfs/BlumenfeldALJoseAM_RNA_2016.pdf`.
#
# (c) Jose Lab, University of Maryland, October 17, 2016.
# Written by Rex Ledesma.
#
##

## import modules
import subprocess
import json
import os
import collections
from pprint import pprint
from datetime import datetime

## global variables
datasets = {}

def main():
    def askIfCorrectDatasets():
        askIfCorrect = raw_input('Are these datasets correct? '
            + 'Type Y for YES and N for NO.\n\t')
        if (askIfCorrect.lower() == 'y'):
            run_workflows()
        elif (askIfCorrect.lower() == 'n'):
            edit_datasets()
        else:
            askIfCorrectDatasets()

    ask_for_arguments()
    askIfCorrectDatasets()

## function that prompts user to add datasets to analyze
def ask_for_arguments():
    ask_args = raw_input('Add galaxy dataset to analyze? Type Y for YES or N '
        + 'for NO, when all datasets are entered. \n\t')
    if (ask_args.lower() == 'y'):
        get_run_workflows_args()
        ask_for_arguments()
    elif (ask_args.lower() == 'n'):
        if (len(datasets) == 0):
            print 'No datasets inputed. Please add galaxy dataset.\n'
            get_run_workflows_args()
            ask_for_arguments()
        else:
            print '\nRunning workflows on:\n'
            pprint(datasets, width = 1)
    else:
        ask_for_arguments()

## prompts user for arguments to run_workflows.py
def get_run_workflows_args():
    name = str(raw_input('What do you want to name your dataset? \n\t')).strip()
    ID = str(raw_input('What is the API ID of your dataset? \n\t')).strip()
    clipper = str(raw_input('Adapter sequence? \n\t')).strip().upper()
    datasets[name] = {'ID': ID, 'label': 'FASTQ', 'clip_sequence': clipper}
    print '\nAdded run:'
    pprint({name: datasets[name]}, width = 1)

def edit_datasets():
    def confirm():
        ok_again = raw_input('Run workflows? Y for YES or N for NO.\n\t')
        if (ok_again.lower() == 'y'):
            run_workflows()
        elif (ok_again.lower() == 'n'):
            ask_edit_again()
        else:
            confirm()

    def ask_edit_again():
        ok = raw_input('Edit again? Y for YES or N for NO.\n\t')
        if (ok.lower() == 'y'):
            edit_datasets()
        elif (ok.lower() == 'n'):
            confirm()
        else:
            ask_edit_again()

    pprint(datasets.keys(), width = 1)
    run = raw_input('Type the name of the dataset you want to edit.\n\t')
    try:
        datasets.pop(run)
        get_run_workflows_args()
        ask_edit_again()
    except KeyError:
        ask_edit_again()

## Asks for template json file, then runs Blumenfeld's run_workflows.py
## script on all the datasets that were entered
def run_workflows():
    print 'Remember to change the API IDs of the workflows in your json file!'
    os.system('read -s -n 1 -p "Press any key to continue..."')
    print

    def ask_for_json_file():
        filename = raw_input('Input path of workflow.json file.\n\t')
        if (not filename.endswith('.json')):
            filename += '.json'
        try:
            return json.load(open(filename, 'r'),
                object_pairs_hook = collections.OrderedDict)
        except Exception as e:
            print str(e)
            print '\tCould not load data from filename. Try again? \n'
            return ask_for_json_file()

    data = ask_for_json_file()

    # configure galaxy url in json file
    url = raw_input('What is your galaxy url?\n\t').strip()
    data['gi_setup']['galaxy_url'] = url

    # configure API id in json file
    api_ID = raw_input('What is your API ID?\n\t').strip()
    data['gi_setup']['api_key'] = api_ID

    nameOfRuns = raw_input('Type the name of your tmux-sessions.\n\t')

    # reminds user to change url in download_datasets_cmd.py
    print 'Remember to change the \'galaxy_url\' and \'api_key\'' \
        + 'in download_datasets_cmd.py!'
    os.system('read -s -n 1 -p "Press any key to continue..."')
    print '\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n'

    # create directory to store json files for all the runs
    workflows_dir = datetime.today().strftime(nameOfRuns + '_%d-%m-%Y_%H:%M:%S')
    os.makedirs(workflows_dir)

    for name in datasets.keys():
        ## copy json file, and change the clipper sequence in copy
        data['workflow_settings']['0'][0]['params'][0]['param_input']   \
            ['clip_source|clip_sequence'] = datasets[name]['clip_sequence']
        with open(workflows_dir + '/workflow_' + name + '.json', 'w') as f2:
            f2.write(json.dumps(data, indent = 2))
            f2.close()

        ## TODO: use tmux within tmux, to analyze datasets one (or two) at a time
            ## since analyzing all at one slows down processing

        ## call run_workflows.py with arguments in detatched window
        subprocess.call(['tmux', 'new', '-s' , nameOfRuns + '_' + name, '-d'])
        try:
            subprocess.call(['tmux' , 'send', '-t', nameOfRuns + '_' + name,
            'source galaxy_run/bin/activate', 'ENTER'])
        except:
            print '<----- Assuming python packages are installed ----->'
        subprocess.call(['tmux', 'send', '-t', nameOfRuns + '_' + name,
            'python run_workflows.py ' + name + ' ' + workflows_dir
            + '/workflow_copy_' + name + '.json ' + datasets[name]['ID']
            + ' ' + datasets[name]['label'], 'ENTER'])

    ## helpful tips for user when running tmux
    print '<----- Running workflows on tmux ----->' \
        + '\n\nUse command:\n\t$tmux ls\nTo show your running workflows.\n\n' \
        + 'To view a workflow in progress, use command\n\t$tmux attach -t' \
        + ' [name]\nwhere [name] is the name of the tmux window.\n\nTo ' \
        + 'exit the tmux window, use escape characters:\n\tctrl b d' \
        + '\n\nTo kill a tmux session, use command\n\t$tmux detach -t [name]' \
        + '\n\nTo kill all tmux sessions, use command\n\t$killall tmux'

if __name__ == "__main__":
    main()
